package ru.myitschool.work.ui.admin

import android.os.Bundle
import android.view.View
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import androidx.navigation.toRoute
import com.squareup.picasso.Picasso
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import ru.myitschool.work.R
import ru.myitschool.work.databinding.FragmentAdminBinding
import ru.myitschool.work.ui.login.LoginDestination
import ru.myitschool.work.ui.main.MainDestination
import ru.myitschool.work.ui.qr.result.QrResultDestination
import ru.myitschool.work.ui.qr.scan.QrScanDestination
import ru.myitschool.work.utils.collectWhenStarted
import ru.myitschool.work.utils.TextChangedListener

@AndroidEntryPoint
class AdminFragment : Fragment(R.layout.fragment_admin) {
    private var _binding: FragmentAdminBinding? = null
    private val binding: FragmentAdminBinding get() = _binding!!
    private val picasso: Picasso by lazy { Picasso.get() }

    private val viewModel: AdminViewModel by viewModels()

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        _binding = FragmentAdminBinding.bind(view)


        val username =
            findNavController().currentBackStackEntry?.toRoute<AdminDestination>()?.username
        username?.let { user ->
            val password =
                findNavController().currentBackStackEntry?.toRoute<AdminDestination>()?.password
            password?.let { password ->

                binding.loginadmin.addTextChangedListener(TextChangedListener {
                    viewModel.onUsernameChanged(it)
                })
                subscribe()
                binding.adminName.text = user
                binding.find.setOnClickListener {
                    login(user, password, binding.loginadmin.text.toString())
                }
                binding.backAdmin.setOnClickListener {
                    findNavController().apply {
                        popBackStack<AdminDestination>(true)
                        navigate(MainDestination(user, password = password))
                    }
                }
            }
        }
    }

    private fun login(username: String, password: String, loginPerson : String) {
        viewModel.loadPersonInfo(username, password, loginPerson) {
            viewModel.state.collectWhenStarted(this) { state ->
                if (state.photo.isNotEmpty()) {
                    picasso.load(state.photo).into(binding.photoAdmin)
                }
                binding.fullnameAdmin.text = state.fullName
                binding.positionAdmin.text = state.position
                binding.lastEntryAdmin.text = state.lastVisit
                setError(state.error)
            }
        }
    }

    private fun subscribe() {
//        viewModel.state.collectWhenStarted(this) { state ->
//            binding.login.isEnabled = state.isLoginEnabled
//            if (state.error != null) {
//                binding.error.visibility = View.VISIBLE
//                binding.error.text = state.error
//            } else {
//                binding.error.visibility = View.GONE
//            }
//        }
//        viewModel.savedUsername.collectWhenStarted(this) { username ->
//            viewModel.savedPassword.collectWhenStarted(this) { password ->
//                if (!username.isNullOrBlank() && !password.isNullOrBlank()) {
//                    login(username, password)
//                }
//            }
//        }
        println("akjshdfg")
    }


    private fun setError(error: String?) {
        val showError = error != null
        val views = listOf(binding.fullnameAdmin, binding.positionAdmin, binding.lastEntryAdmin, binding.photoAdmin)
        views.forEach { it.visibility = if (showError) View.GONE else View.VISIBLE}
        binding.errorAdmin.visibility = if (showError) View.VISIBLE else View.GONE
        binding.block.visibility = if (showError) View.GONE else View.VISIBLE
        binding.errorAdmin.text = error ?: ""
    }

    override fun onDestroyView() {
        _binding = null
        super.onDestroyView()
    }
}